"""Authentication methods for local domain on win32"""

import sys, string
from zLOG import LOG, WARNING
import win32api
import win32net, win32netcon
import win32security

def get_local_ntdom():
    """Return the local NT Domain
    If the local machine is a domain controller, returns the current domain
    If the local machine is a standalone server, returns the machine name
    """
    infos = win32net.NetServerGetInfo(None, 101)
    type = infos['type']
    if (type & win32netcon.SV_TYPE_DOMAIN_CTRL) or (type & win32netcon.SV_TYPE_DOMAIN_BAKCTRL):
        local_ntdom = win32api.GetDomainName()
        # XXX - there *has* to be a way to get the domain
        # this computer is a controller for
    else:
        local_ntdom = infos['name']
    return string.lower(str(local_ntdom))

def get_usernames(ntdom=None):
    """Return the users of the local NT Domain
    If ntdom is not None, usernames are prefixed with it

    Beware! returned usernames are not lowercase!
    """
    resume = 0
    usernames = []

    # ask the domain controller
    while 1:
        users, junk, resume = win32net.NetUserEnum(None, 0, win32netcon.FILTER_NORMAL_ACCOUNT, resume)
        ntnames = map(lambda user: string.lower(str(user['name'])), users)
        if ntdom is not None:
            ntnames = map(lambda ntname, ntdom=ntdom: ntdom + '\\' + ntname, ntnames)
        usernames.extend(ntnames)
        if not resume:
            break
    return usernames

def get_userinfo(ntname, ntdom=None):
    """Return the full name and the comment of a local user
    ntdom is ignored
    """
    try:
        userinfo = win32net.NetUserGetInfo(None, ntname, 20)
    except win32api.error:
        return None
    return {'full_name': str(userinfo['full_name']),
            'comment': str(userinfo['comment']),
            }

def authenticate_user(ntname, ntdom, password):
    token = None
    try:
        try:
            token = win32security.LogonUser(
                ntname,
                ntdom,
                password,
                win32security.LOGON32_LOGON_NETWORK,
                win32security.LOGON32_PROVIDER_DEFAULT)
            return 1
        except:
            LOG('NTUserFolder',
                WARNING,
                'Error trying to authenticate %s\%s' % (ntdom, ntname),
                'LogonUser raised an exception\n',
                sys.exc_info())
    finally:
        if token:
            win32api.CloseHandle(token)
    return 0

